/*!
 * jQuery UI Timepicker 1.8.23.0
 *
 * Copyright(C) 2012 Takuya Kurosawa
 *
 * License:
 *  MIT License (http://www.opensource.org/licenses/mit-license.php
 *
 * Depends:
 *  jquery.ui.core.js (1.8.23)
 *  jquery.ui.widget.js (1.8.23)
 *
 * Original code:
 *  jQuery ui.timepickr (http://haineault.com/)
 */

(function($) {

$.widget('ui.timepicker', {
	widgetEventPrefix: "timepicker",

	options:
	{
		convention:  24, // 24, 12
		trigger:     'click',
		format12:    'hh:mm z',
		format24:    'hh:mm',
		prefix:      ['am', 'pm'],
		suffix:      ['am', 'pm'],
		prefixVal:   false,
		suffixVal:   true,
		rangeHour12: [1,2,3,4,5,6,7,8,9,10,11,12],
		rangeHour24: [[0,1,2,3,4,5,6,7,8,9,10,11], [12,13,14,15,16,17,18,19,20,21,22,23]],
		rangeMin:    [0,15,30,45],
		rangeSec:    [0,15,30,45],
		corners:     false,

		core:        true,
		hours:       true,
		minutes:     true,
		seconds:     false,
		val:         false,
		updateLive:  true,
		resetOnBlur: true,
		keyboardnav: true,
		handle:      false,
		handleEvent: 'click'
	},

	plugins: {},

	_init: function()
	{
		this._dom =
		{
			menu: $('<div class="ui-helper-reset ui-timepicker ui-widget" />'),
			row:  $('<div class="ui-helper-reset ui-timepicker ui-widget" />')
		};

		this._trigger('initialize');
		this._trigger('initialized');
	},

	_trigger: function(type, e, ui)
	{
		var ui = ui || this;
		$.ui.plugin.call(this, type, [e, ui]);
		return $.Widget.prototype._trigger.call(this, type, e, ui);
	},

	_createButton: function(i, number, className)
	{
		var o = number ? ('00' + i).slice(-2) : i;
		return $('<li class="ui-timepicker-button"><span class="ui-state-default">' + o + '</span></li>').data('id', i)
				.bind
				(
					'mouseover',
					function()
					{
						$(this).siblings().find('span').removeClass('ui-state-hover').end().end().find('span').addClass('ui-state-hover');
					}
				);
	},

	_addRow: function(range, number, className, insertAfter)
	{
		var ui  = this;

		var row = $('<ol class="ui-timepicker-row ui-helper-clearfix" />')
					.bind
					(
						'mouseover',
						function()
						{
							$(this).next().show();
						}
					);

        $.each
		(
			range, 
			function(idx, val)
			{
				ui._createButton(val, number).appendTo(row);
			}
		);

		if (className)
			$(row).addClass(className);

		if (this.options.corners)
			row.find('span').addClass('ui-corner-'+ this.options.corners);

		if (insertAfter)
			row.insertAfter(insertAfter);
		else
			ui._dom.menu.append(row);

		return row;
	},

	_setVal: function(val)
	{
		val = val || this._getVal();
		this.element.data('timepicker.initialValue', val);
		this.element.val(this._formatVal(val));
		if(this._dom.menu.is(':hidden'))
			this.element.trigger('change');
	},

	_getVal: function()
	{
		var ols = this._dom.menu.find('ol');

		var get = function(unit)
		{
			var u = ols.filter('.'+unit).find('.ui-state-hover:first').text();
			return u || ols.filter('.'+unit+'li:first span').text();
		}

		return {
			h: get('hours'),
			m: get('minutes'),
			s: get('seconds'),
			a: get('prefix'),
			z: get('suffix'),
			f: this.options['format'+ this.c],
			c: this.c
		};
	},

	_formatVal: function(ival)
	{
		var format = this.options.convention === 12 ? this.options.format12 : this.options.format24;

		var lookAhead = function(match)
		{
			var matches = (iFormat + 1 < format.length && format.charAt(iFormat + 1) == match);
			if (matches)
				iFormat++;
			return matches;
		};

		var formatNumber = function(match, value)
		{
			if (lookAhead(match))
				return ('00' + value).slice(-2);
			else
				return '' + value;
		};

		var result = '';
		var literal = false;
		for (var iFormat = 0; iFormat < format.length; iFormat++) {
			if (literal)
			{
				if (format.charAt(iFormat) == "'" && !lookAhead("'"))
					literal = false;
				else
					result += format.charAt(iFormat);
			}
			else
			{
				switch (format.charAt(iFormat))
				{
					case 'h':
						result += formatNumber('h', ival.h);
						break;
					case 'm':
						result += formatNumber('m', ival.m);
						break;
					case 's':
						result += formatNumber('s', ival.s);
						break;
					case 'z':
						result += ival.z;
						break;
					case "'":
						if (lookAhead("'"))
							result += "'";
						else
							literal = true;
						break;
					default:
						result += format.charAt(iFormat);
				}
			}
		}

		return result;
	},

    blur: function()
	{
		return this.element.blur();
	},

	focus: function()
	{
		return this.element.focus();
	},

	show: function()
	{
		this._trigger('show');
		this.element.trigger(this.options.trigger);
	},

	hide: function()
	{
		this._trigger('hide');
		this._dom.menu.hide();
	}
});

$.ui.plugin.add('timepicker', 'core', {
	initialized: function(e, ui)
	{
		var menu = ui._dom.menu;
		var pos  = ui.element.position();

		menu.insertAfter(ui.element).css('left', pos.left);

		if (!$.support.boxModel)
			menu.css('margin-top', ui.element.height() + 8);
        
		ui.element
		.bind
		(
			ui.options.trigger,
			function()
			{
				ui._dom.menu.show();
				ui._dom.menu.find('ol:first').show();
				ui._trigger('focus');
				if (ui.options.trigger != 'focus')
					ui.element.focus();
				ui._trigger('focus');
            }
		)
		.bind
		(
			'blur',
			function()
			{
				ui.hide();
				ui._trigger('blur');
			}
		);

		menu.find('li').bind('mouseover.timepicker', function() {ui._trigger('refresh');});
	},

	refresh: function(e, ui)
	{
		// Realign each menu layers
		ui._dom.menu.find('ol').each(function(){
			var p = $(this).prev('ol');
			try { // .. to not fuckup IE
				$(this).css('left', p.position().left + p.find('.ui-state-hover').position().left);
			} catch(e) {};
		});
	}
});

$.ui.plugin.add('timepicker', 'hours', {
	initialize: function(e, ui)
	{
		if (ui.options.convention === 24)
		{
			// prefix is required in 24h mode
			ui._dom.prefix = ui._addRow(ui.options.prefix, false, 'prefix'); 

			// split-range
			if ($.isArray(ui.options.rangeHour24[0]))
			{
				var range = [];
				$.merge(range, ui.options.rangeHour24[0]);
				$.merge(range, ui.options.rangeHour24[1]);
				ui._dom.hours = ui._addRow(range, true, 'hours');
				ui._dom.hours.find('li').slice(ui.options.rangeHour24[0].length, -1).hide();
				var lis   = ui._dom.hours.find('li'); 

				var show = [
					function()
					{
						lis.slice(ui.options.rangeHour24[0].length).hide().end()
						.slice(0, ui.options.rangeHour24[0].length).show()
						.filter(':visible:first').trigger('mouseover');
					},
					function()
					{
						lis.slice(0, ui.options.rangeHour24[0].length).hide().end()
						.slice(ui.options.rangeHour24[0].length).show()
						.filter(':visible:first').trigger('mouseover');
					}
				];

				ui._dom.prefix.find('li')
				.bind
				(
					'mouseover.timepicker',
					function()
					{
						var index = ui._dom.menu.find('.prefix li').index(this);
						show[index].call();
					}
				);
			}
			else
			{
				ui._dom.hours = ui._addRow(ui.options.rangeHour24, true, 'hours');
				ui._dom.hours.find('li').slice(12, -1).hide();
			}
		}
		else
		{
			ui._dom.hours  = ui._addRow(ui.options.rangeHour12, true, 'hours');
			// suffix is required in 12h mode
			ui._dom.suffix = ui._addRow(ui.options.suffix, false, 'suffix'); 
		}
	}
});

$.ui.plugin.add('timepicker', 'minutes', {
	initialize: function(e, ui)
	{
		var p = ui._dom.hours && ui._dom.hours || false;
		ui._dom.minutes = ui._addRow(ui.options.rangeMin, true, 'minutes', p);
	}
});

$.ui.plugin.add('timepicker', 'seconds', {
	initialize: function(e, ui)
	{
		var p = ui._dom.minutes && ui._dom.minutes || false;
		ui._dom.seconds = ui._addRow(ui.options.rangeSec, true, 'seconds', p);
	}
});

$.ui.plugin.add('timepicker', 'val', {
	initialized: function(e, ui)
	{
		ui._setVal(ui.options.val);
	}
});

$.ui.plugin.add('timepicker', 'updateLive', {
	refresh: function(e, ui)
	{
		ui._setVal();
	}
});

$.ui.plugin.add('timepicker', 'resetOnBlur', {
	initialized: function(e, ui)
	{
		ui.element.data('timepicker.initialValue', ui._getVal());
		ui._dom.menu.find('li > span')
		.bind
		(
			'mousedown.timepicker',
			function()
			{
				ui.element.data('timepicker.initialValue', ui._getVal()); 
			}
		);
	},

	blur: function(e, ui)
	{
		ui._setVal(ui.element.data('timepicker.initialValue'));
	}
});

$.ui.plugin.add('timepicker', 'handle', {
	initialized: function(e, ui)
	{
		$(ui.options.handle).bind(ui.options.handleEvent + '.timepicker', function(){ui.show();});
	}
});

$.ui.plugin.add('timepicker', 'keyboardnav', {
	initialized: function(e, ui)
	{
		ui.element
		.bind
		(
			'keydown',
			function(e)
			{
				if ($.keyIs('enter', e))
				{
					ui._setVal();
					ui.blur();
				}
				else if ($.keyIs('escape', e))
					ui.blur();
			}
		);
	}
});

})(jQuery);
